"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Koa = require("koa");
const bodyParser = require("koa-bodyparser");
const _constants_1 = require("../constants");
const _utils_1 = require("../utils");
const _services_1 = require("../services");
const _decorators_1 = require("../decorators");
const lenneth_setting_1 = require("./lenneth-setting");
class LennethApplication {
    constructor() {
        const _settingParams = this.lennethSetting.getMetadata(this);
        if (_settingParams) {
            this._setSettings(_settingParams);
        }
    }
    _setSettings(settings) {
        this.lennethSetting.setMap(settings);
    }
    _callHook(key, elseFn = new Function(), ...args) {
        const self = this;
        if (key in self) {
            return self[key](...args);
        }
        return elseFn();
    }
    async _loadInterceptor() {
        let Interceptor = lenneth_setting_1.LennethSetting.serverSettingMap.get("interceptor");
        if (Interceptor) {
            this._loadMiddleware(Interceptor);
        }
    }
    async _loadBodyParser() {
        return new Promise((res, err) => {
            this.app.use(bodyParser());
            res();
        });
    }
    async _loadRouters() {
        let imports = lenneth_setting_1.LennethSetting.serverSettingMap.get("imports");
        return new Promise((res, err) => {
            this.routerService.joinControllerPath(imports);
            this.routerService.loadRouter(this.app);
            res();
        });
    }
    async _errorMiddleware() {
        let serverSettingsMap = lenneth_setting_1.LennethSetting.serverSettingMap;
        let errorMiddleware = serverSettingsMap.get("globalError");
        if (errorMiddleware) {
            let errorMiddlewarFun = new errorMiddleware().use;
            let asyncMiddle = _utils_1.toErrorAsyncMiddleware(errorMiddleware.prototype, errorMiddlewarFun, errorMiddlewarFun[_constants_1.LENNETH_ERROR_NAME], this.paramsService.paramsToErrorList);
            this.app.on("error", asyncMiddle);
        }
    }
    async _responseMiddleware() {
        let serverSettingsMap = lenneth_setting_1.LennethSetting.serverSettingMap;
        let responseMiddleware = serverSettingsMap.get("response");
        if (responseMiddleware) {
            this._loadMiddleware(responseMiddleware);
        }
    }
    _loadMiddleware(middlewareClass) {
        if (middlewareClass) {
            let middlewareFun = new middlewareClass().use;
            let asyncMiddle = _utils_1.toAsyncMiddleware(middlewareClass.prototype, middlewareFun, middlewareFun[_constants_1.LENNETH_MIDDLEWARE_NAME], this.paramsService.paramsToList);
            this.app.use(asyncMiddle);
        }
    }
    async _startServer() {
        let { hostname, port } = this.lennethSetting.getHttpPort();
        return new Promise((res, err) => {
            this.app.listen(port, hostname);
            this.logger.info(`app start ${port}`);
            res();
        });
    }
    use(middleware) {
        this.app.use(middleware);
        return this;
    }
    async start() {
        const startTime = new Date();
        try {
            await this._callHook("$onInit");
            await this._loadInterceptor();
            await this._callHook("$onMountingMiddlewares", undefined, this.app);
            await this._loadBodyParser();
            await this._callHook("$onRoutesInit", undefined, this.app);
            await this._responseMiddleware();
            await this._loadRouters();
            await this._callHook("$afterRoutesInit", undefined, this.app);
            await this._errorMiddleware();
            await this._startServer();
            await this._callHook("$onReady");
        }
        catch (error) {
            this._callHook("$onServerInitError", undefined, error);
            Promise.reject(error);
            throw error;
        }
    }
}
tslib_1.__decorate([
    _decorators_1.Autowired(),
    tslib_1.__metadata("design:type", Koa)
], LennethApplication.prototype, "app", void 0);
tslib_1.__decorate([
    _decorators_1.Autowired(),
    tslib_1.__metadata("design:type", lenneth_setting_1.LennethSetting)
], LennethApplication.prototype, "lennethSetting", void 0);
tslib_1.__decorate([
    _decorators_1.Autowired(),
    tslib_1.__metadata("design:type", _services_1.RouterService)
], LennethApplication.prototype, "routerService", void 0);
tslib_1.__decorate([
    _decorators_1.Autowired(),
    tslib_1.__metadata("design:type", _services_1.ParamsService)
], LennethApplication.prototype, "paramsService", void 0);
tslib_1.__decorate([
    _decorators_1.Autowired(),
    tslib_1.__metadata("design:type", _services_1.LoggerService)
], LennethApplication.prototype, "logger", void 0);
exports.LennethApplication = LennethApplication;
//# sourceMappingURL=lenneth-application.js.map